/**
 * @author Rafael Ostertag <rafael.ostertag@math.uzh.ch>
 */

/* global $ */

if (!QfqNS) {
    var QfqNS = {};
}

(function (n) {
    'use strict';

    n.Form = function (formId) {
        this.formId = formId;

        if (!document.forms[this.formId]) {
            throw new Error("Form '" + formId + "' does not exist.");
        }

        this.formChanged = false;
        this.userFormChangeHandlers = new QfqNS.Helper.FunctionList();
        this.userResetHandlers = new QfqNS.Helper.FunctionList();
        this.userSubmitSuccessHandlers = new QfqNS.Helper.FunctionList();
        this.userSubmitFailureHandlers = new QfqNS.Helper.FunctionList();

        this.$form = $(document.forms[this.formId]);
        this.$form.on("change", this.changeHandler.bind(this));
    };

    /**
     *
     * @param event
     *
     * @private
     */
    n.Form.prototype.changeHandler = function (event) {
        this.formChanged = true;
        this.userFormChangeHandlers.call(this);
    };

    n.Form.prototype.addChangeHandler = function (callback) {
        this.userFormChangeHandlers.addFunction(callback);
    };

    n.Form.prototype.addResetHandler = function (callback) {
        this.userResetHandlers.addFunction(callback);
    };

    n.Form.prototype.addSubmitSuccessHandler = function (callback) {
        this.userSubmitSuccessHandlers.addFunction(callback);
    };

    n.Form.prototype.addSubmitFailureHandler = function (callback) {
        this.userSubmitFailureHandlers.addFunction(callback);
    };

    n.Form.prototype.getFormChanged = function () {
        return this.formChanged;
    };

    n.Form.prototype.resetFormChanged = function () {
        this.formChanged = false;
        this.userResetHandlers.call(this);
    };

    n.Form.prototype.submitTo = function (to) {
        $.post(to, this.$form.serialize())
            .done(this.ajaxSuccessHandler.bind(this))
            .fail(this.submitFailureHandler.bind(this));
    };

    /**
     *
     * @param data
     * @param textStatus
     * @param jqXHR
     *
     * @private
     */
    n.Form.prototype.ajaxSuccessHandler = function (data, textStatus, jqXHR) {
        this.userSubmitSuccessHandlers.call(this, data, textStatus);
    };

    /**
     *
     *
     * @private
     */
    n.Form.prototype.submitFailureHandler = function (jqXHR, textStatus, errorThrown) {
        this.userSubmitFailureHandlers.call(this, textStatus, jqXHR, errorThrown);
    };

})(QfqNS);
